from argparse import Namespace
from typing import Literal

from evoprompt.cli import argument_parser
from evoprompt.models import LLMModel

# make sure to run definitions of subclasses of Task first
from evoprompt.task.task import EvaluationStrategyKey, Task
from evoprompt.task.question_answering import QuestionAnswering
from evoprompt.task.text_classification import TextClassification
from evoprompt.task.sentiment_analysis import SentimentAnalysis
from evoprompt.task.topic_classification import AGNews, TREC
from evoprompt.task.subjectivity_classification import Subj

from evoprompt.utils import get_all_subclasses

# at this point, we assume that all subclasses of Task have been defined
tasks = {
    task.shorthand: task
    for task in get_all_subclasses(Task)
    if hasattr(task, "shorthand")
}


def get_task(name: str, evaluation_model: LLMModel, **options):
    if name not in tasks:
        raise ValueError("Model %s does not exist", name)
    return tasks[name](evaluation_model, **options)


argument_parser.add_argument("--debug", "-d", action="store_true", default=None)
argument_group = argument_parser.add_argument_group("Task arguments")
argument_group.add_argument("--use-grammar", "-g", action="store_true")
argument_group.add_argument(
    "--evaluation-strategy",
    choices=EvaluationStrategyKey.__args__,
    default="simple",
)
argument_group.add_argument(
    "--task", "-t", type=str, required=True, choices=tasks.keys()
)
